#!/usr/bin/python
# -*- coding: UTF-8 -*-
from pprint import pformat
from dli.rest import PkgResourceInfo
from dli.rest import GroupResourceInfo


class PackageResource(object):
    def __init__(self, pkg_res_info):
        """
        Init the PackageResource instance

        :param PkgResourceInfo pkg_res_info:
        """
        self._name = pkg_res_info.resource_name
        self._type = pkg_res_info.resource_type
        self._create_time = pkg_res_info.create_time
        self._update_time = pkg_res_info.update_time
        self._status = pkg_res_info.status

    @property
    def resource_name(self):
        return self._name

    @property
    def resource_type(self):
        return self._type

    @property
    def resource_create_time(self):
        return self._create_time

    @property
    def resource_update_time(self):
        return self._update_time

    @property
    def resource_status(self):
        return self._status

    def to_dict(self):
        result = {
            'resource_name': self._name,
            'resource_type': self._type,
            'resource_create_time': self._create_time,
            'resource_update_time': self._update_time,
            'resource_status': self._status
        }
        return result

    def to_str(self):
        return pformat(self.to_dict())

    def __repr__(self):
        return self.to_str()


class GroupResource(object):
    def __init__(self, group_res_info):
        """
        Init the GroupResource instance

        :param GroupResourceInfo group_res_info:
        """
        self._group_name = group_res_info.group_name
        self._status = group_res_info.status
        self._resources = group_res_info.resources  # type: list[str]
        self._create_time = group_res_info.create_time
        self._update_time = group_res_info.update_time
        self._details = None    # type: list[PackageResource]

    @property
    def group_name(self):
        return self._group_name

    @property
    def resource_status(self):
        return self._status

    @property
    def resources(self):
        return self._resources

    @property
    def create_time(self):
        return self._create_time

    @property
    def update_time(self):
        return self._update_time

    @property
    def details(self):
        return self.details

    def to_dict(self):
        result = {
            'group_name': self._group_name,
            'status': self._status,
            'resources': self._resources,
            'create_time': self._create_time,
            'update_time': self._update_time,
            'details': self._details
        }
        return result

    def to_str(self):
        return pformat(self.to_dict())

    def __repr__(self):
        return self.to_str()


class Resources(object):
    def __init__(self, pkg_resources=None, group_resources=None):
        """
        Init the GroupResource instance

        :param list[PackageResource] pkg_resources:
        :param list[GroupResource] group_resources:
        """
        self._pkg_resources = pkg_resources
        self._group_resources = group_resources

    @property
    def package_resources(self):
        return self._pkg_resources

    @property
    def group_resources(self):
        return self._group_resources

    def to_dict(self):
        result = {
            'package_resources': self._pkg_resources,
            'group_resources': self._group_resources,
        }
        return result

    def to_str(self):
        return pformat(self.to_dict())

    def __repr__(self):
        return self.to_str()

