#!/usr/bin/python
# -*- coding: UTF-8 -*-
import time

from dli.table import TableSchema
from dli.column import Column
from dli.exception import DliException
from dli.dli_client import DliClient
from dli.job import SparkJobInfo

PROXY = 'https://ip:port'


def init_aksk_dli_client():
    auth_mode = "aksk"
    region = 'cn-north-1'
    project_id = 'xxxx'
    ak = 'xxxx'
    sk = 'xxxx'

    dli_client = DliClient(auth_mode=auth_mode, region=region, project_id=project_id,
                           ak=ak, sk=sk)
    return dli_client


def init_token_dli_client():
    auth_mode = "token"
    region = 'cn-north-1'
    project_id = 'xxxx'
    account = 'huaweicloud_account'
    user = 'iam_user_name'
    password = 'xxxx'

    dli_client = DliClient(auth_mode=auth_mode, region=region, project_id=project_id,
                           account=account, user=user, password=password)
    return dli_client


def list_all_sql_jobs(dli_client):
    try:
        sql_jobs = dli_client.list_sql_jobs()
    except DliException as e:
        print(e)
        return
    for sql_job in sql_jobs:
        print(sql_job)


def cancel_sql(dli_client, job_id):
    try:
        dli_client.cancel_sql(job_id)
    except DliException as e:
        print(e)
        return


def del_spark_batch(dli_client, batch_id):
    try:
        resp = dli_client.del_spark_batch_job(batch_id)
        print(resp.msg)
    except DliException as e:
        print(e)
        return


def list_all_queues(dli_client):
    try:
        queues = dli_client.list_queues()
    except DliException as e:
        print(e)
        return

    for queue in queues:
        print(queue.name)


def create_db(dli_client):
    try:
        db = dli_client.create_database('db_for_test')
    except DliException as e:
        print(e)
        return

    print(db)

def create_db_with_eps(dli_client):
    try:
        db = dli_client.create_database_with_eps('db_for_test', "epId")
    except DliException as e:
        print(e)
        return

    print(db)


def list_all_dbs(dli_client):
    try:
        dbs = dli_client.list_databases()
    except DliException as e:
        print(e)
        return

    for db in dbs:
        print(db)


def create_dli_tbl(dli_client, db_name, tbl_name):
    cols = [
        Column('col_1', 'string'),
        Column('col_2', 'string'),
        Column('col_3', 'smallint'),
        Column('col_4', 'int'),
        Column('col_5', 'bigint'),
        Column('col_6', 'double'),
        Column('col_7', 'decimal(10,0)'),
        Column('col_8', 'boolean'),
        Column('col_9', 'date'),
        Column('col_10', 'timestamp')
    ]
    sort_cols = ['col_1']
    tbl_schema = TableSchema(tbl_name, cols, sort_cols)
    try:
        table = dli_client.create_dli_table(db_name, tbl_schema)
    except DliException as e:
        print(e)
        return

    print(table)


def create_obs_tbl(dli_client, db_name, tbl_name):
    cols = [
        Column('col_1', 'string'),
        Column('col_2', 'string'),
        Column('col_3', 'smallint'),
        Column('col_4', 'int'),
        Column('col_5', 'bigint'),
        Column('col_6', 'double'),
        Column('col_7', 'decimal(10,0)'),
        Column('col_8', 'boolean'),
        Column('col_9', 'date'),
        Column('col_10', 'timestamp')
    ]
    tbl_schema = TableSchema(tbl_name, cols)
    try:
        table = dli_client.create_obs_table(db_name, tbl_schema,
                                            's3a://bucket/obj',
                                            'csv')
    except DliException as e:
        print(e)
        return

    print(table)


def list_all_tbls(dli_client, db_name):
    try:
        tbls = dli_client.list_tables(db_name, with_detail=True)
    except DliException as e:
        print(e)
        return

    for tbl in tbls:
        print(tbl.name)


def list_tbls_by_page(dli_client, db_name, page_size, current_page):
    try:
        tbls = dli_client.list_tables_page(db_name, page_size, current_page, with_detail=True)
    except DliException as e:
        print(e)
        return

    for tbl in tbls:
        print(tbl.name)



def import_data(dli_client, db_name, tbl_name, queue_name):
    try:
        job_id, status = \
            dli_client.import_table(tbl_name, db_name,
                                    's3a://bucket/obj/data.csv',
                                    'csv', queue_name=queue_name)
    except DliException as e:
        print(e)
        return

    print(job_id)
    print(status)


def export_data(dli_client, db_name, tbl_name, queue_name):
    try:
        job_id, status = dli_client.export_table(tbl_name, db_name,
                                                 's3a://bucket/obj',
                                                 queue_name=queue_name)
    except DliException as e:
        print(e)
        return

    print(job_id)
    print(status)


def run_sql(dli_client, db_name, queue_name):
    # execute SQL
    try:
        sql_job = dli_client.execute_sql('select * from tbl_dli_for_test', db_name)
        result_set = sql_job.get_result()
    except DliException as e:
        print(e)
        return

    if result_set.row_count == 0:
        return

    for row in result_set:
        print(row)

    # export the query result to obs
    try:
        status = sql_job.export_result('s3a://bucket/obj',
                                       queue_name=queue_name)
    except DliException as e:
        print(e)
        return

    print(status)


def delete_tbls(dli_client, db_name):
    try:
        tbls = dli_client.list_tables(db_name)
        for tbl in tbls:
            dli_client.delete_table(db_name, tbl.name)
    except DliException as e:
        print(e)
        return


def delete_db(dli_client, db_name):
    try:
        dli_client.delete_database(db_name)
    except DliException as e:
        print(e)
        return


def async_run_sql(dli_client, queue_name, db_name):
    try:
        sql_job = dli_client.async_execute_sql('select * from tbl_dli_for_test',
                                               db_name, queue_name)
        while True:
            time.sleep(3)
            job_status = sql_job.get_status()
            if job_status in ['FINISHED', 'CANCELLED', 'FAILED']:
                break

        if job_status != 'FINISHED':
            print('SQL run fail')
            return

        result_set = sql_job.get_result()
        schema = sql_job.get_schema()

    except DliException as e:
        print(e)
        return

    print(schema)

    if result_set.row_count == 0:
        return

    for row in result_set:
        print(row)


def list_resources(dli_client):
    try:
        resources = dli_client.list_resources()
    except DliException as e:
        print(e)
        return

    for resources_info in resources.package_resources:
        print('Package resource name:' + resources_info.resource_name)

    for group_resource in resources.group_resources:
        print('Group resource name:' + group_resource.group_name)


def upload_resource(dli_client, kind, obs_jar_path, group_name):
    try:
        dli_client.upload_resource(kind, obs_jar_path, group_name)
    except DliException as e:
        print(e)
        return


def submit_spark_batch_job(dli_client, batch_queue_name, batch_job_info):
    try:
        batch_job = dli_client.submit_spark_batch_job(batch_queue_name, batch_job_info)
    except DliException as e:
        print(e)
        return

    print(batch_job.job_id)
    while True:
        time.sleep(3)
        job_status = batch_job.get_job_status()
        print('Job status: {0}'.format(job_status))
        if job_status == 'dead' or job_status == 'success':
            break

    logs = batch_job.get_driver_log(500)
    for log_line in logs:
        print(log_line)


def main():
    dli_client = init_aksk_dli_client()

    # This is an example for show how to use dli python sdk to implement sql application.
    sql_queue_name = 'queue_for_sql_test'
    db_name = 'db_for_test'
    dli_tbl_name = 'tbl_dli_for_test'
    obs_tbl_name = 'tbl_obs_for_test'

    list_all_queues(dli_client)

    create_db(dli_client)

    create_db_with_eps(dli_client)

    list_all_dbs(dli_client)

    create_dli_tbl(dli_client, db_name, dli_tbl_name)

    create_obs_tbl(dli_client, db_name, obs_tbl_name)

    list_all_tbls(dli_client, db_name)

    import_data(dli_client, db_name, dli_tbl_name, sql_queue_name)

    export_data(dli_client, db_name, dli_tbl_name, sql_queue_name)

    run_sql(dli_client, db_name, sql_queue_name)

    async_run_sql(dli_client, db_name, sql_queue_name)

    delete_tbls(dli_client, db_name)

    delete_db(dli_client, db_name)

    # This is an example for show how to use dli python sdk to implement sql application.
    batch_queue_name = 'queue_for_batch_test'
    kind = 'jar'
    obs_jar_path = 'https://a-sdk-test.obs.cn-north-1.myhuaweicloud.com/dli-spark-submit-resources/spark-examples_2.11-2.1.0.luxor.jar'
    group_name = 'TestGroup'
    args = ['10']

    list_resources(dli_client)

    upload_resource(dli_client, kind, obs_jar_path, group_name)

    batch_job_info = SparkJobInfo('spark-examples_2.11-2.1.0.luxor.jar',
                                  app_name='spark_pi',
                                  class_name='org.apache.spark.examples.SparkPi',
                                  args=args)
    submit_spark_batch_job(dli_client, batch_queue_name, batch_job_info)


if __name__ == '__main__':
    main()
