#!/usr/bin/python
# -*- coding: UTF-8 -*-
import time

from dli.table import TableSchema
from dli.column import Column
from dli.exception import DliException
from dli.dli_client import DliClient
from dli.job import SparkJobInfo
from obs import ObsClient
import uuid

PROXY = 'https://ip:port'
OBS_ENDPOINT = "obs.cn-north-1.myhuaweicloud.com"
Bucket = "demo"

def init_aksk_dli_client():
    auth_mode = "aksk"
    region = 'cn-north-1'
    project_id = 'xxxx'
    ak = 'xxxx'
    sk = 'xxxx'

    dli_client = DliClient(auth_mode=auth_mode, region=region, project_id=project_id,
                           ak=ak, sk=sk)
    return dli_client


def init_token_dli_client():
    auth_mode = "token"
    region = 'cn-north-1'
    project_id = 'xxxx'
    account = 'huaweicloud_account'
    user = 'iam_user_name'
    password = 'xxxx'

    dli_client = DliClient(auth_mode=auth_mode, region=region, project_id=project_id,
                           account=account, user=user, password=password)
    return dli_client


def submit_spark_batch_python_job(dli_client, **kwargs):
    all_params = ['batch_queue_name', 'python_info', 'obs_endpoint', 'bucket_name', 'path', 'ak', 'sk'
                  'file_name', 'local_file']

    batch_queue_name = kwargs.get('batch_queue_name')
    python_info = kwargs.get('python_info')
    obs_endpoint = 'https://' + kwargs.get('obs_endpoint')
    bucket_name = kwargs.get('bucket_name')
    path = kwargs.get('path')
    ak = kwargs.get('ak')
    sk = kwargs.get('sk')
    file_name = kwargs.get('file_name')
    local_file = kwargs.get('local_file')

    if file_name is None or file_name == "":
        file_name = str(uuid.uuid1()) + '.py'

    if path is None or path == "/" or path == "":
        object_key = file_name
    else:
        object_key = path + '/' + file_name

    # Set the python file path.
    obs_file = 'obs://' + bucket_name + '/' + object_key
    batch_job_info = SparkJobInfo('spark-examples_2.11-2.1.0.luxor.jar',
                                  app_name='spark_pi',
                                  class_name='org.apache.spark.examples.SparkPi',
                                  args=['10'],
                                  py_files=[obs_file])
    try:
        if ak is None or sk is None:
            obs_client = ObsClient(access_key_id=dli_client.dli_info.ak, secret_access_key=dli_client.dli_info.sk,
                                   server=obs_endpoint)
        else:
            obs_client = ObsClient(access_key_id=ak, secret_access_key=sk, server=obs_endpoint)

        # Upload the python file to obs bucket.
        if local_file is None or local_file == "":
            resp = obs_client.putContent(bucket_name, object_key, python_info)
            if resp.status < 300:
                print('Create object ' + object_key + ' successfully!\n')
            obs_client.close()
        else:
            resp = obs_client.uploadFile(bucket_name, object_key, local_file)
            if resp.status < 300:
                print('Create object ' + object_key + ' successfully!\n')
            obs_client.close()
        batch_job = dli_client.submit_spark_batch_job(batch_queue_name, batch_job_info)
    except DliException as e:
        raise DliException(api_exception=e)

    print(batch_job.job_id)
    while True:
        time.sleep(3)
        job_status = batch_job.get_job_status()
        print('Job status: {0}'.format(job_status))
        if job_status == 'dead' or job_status == 'success':
            break

    logs = batch_job.get_driver_log(500)
    for log_line in logs:
        print(log_line)


def main():
    dli_client = init_token_dli_client()

    # This is an example for show how to use dli python sdk to implement sql application.
    batch_queue_name = 'XXXX'
    bucket_name = 'demo'
    python_info = 'python'
    ak = 'XXXX'
    sk = 'XXXX'

    submit_spark_batch_python_job(dli_client,
                                  batch_queue_name=batch_queue_name,
                                  python_info=python_info,
                                  obs_endpoint=OBS_ENDPOINT,
                                  bucket_name=bucket_name,
                                  ak=ak,
                                  sk=sk)

if __name__ == '__main__':
    main()
